#!/bin/sh
scriptname=`basename $0`

# udhcpc script edited by O.Phillips 27/11/2006
#
# Uses the following storage files
# dhcp.eth0.sh  - Settings supplied by DHCP
# active.eth0.act.sh - Setting applied to interface and notified to framework
#

. /Opera_Deploy/networking/lib.sh       # pick up library functions

testing=0

if [ -z "$interface" ]; then
    interface="eth0"
fi

baseInit;

#Now the script proper...

rm -f $eventLog

#Catasrophic no comms pipe to framework
touch $eventLog
if [ ! -e $fwEventPipe ]; then
    echo "$0: $fwEventPipe Not Available exiting" >> $eventLog
    exit
fi


[ -z "$1" ] && echo "Error: should be called from udhcpc" && exit 1


case "$1" in
    deconfig)
        # Have lost lease; deconfigure interface and delete settings, if not enableDhcpLeaseReuse
        # /data/networking/enableDhcpLeaseReuse ... if file exists, then the feature is switched on
        # /tmp/isBound                          ... if file exists, then there was a valid lease negotated after boot and
        #                                           the interface is still configured
        # /tmp/dhcpExpiredLeaseReused           ... if file exists, then there was a valid lease negotated after boot or
        #                                           there was a valid lease negotated before boot without sending a release
        # /tmp/addressWatchdog.pid              ... if file exists, the addressWatchdog is running

        if [ -f /data/networking/enableDhcpLeaseReuse ] && [ -f /tmp/isBound ] && [ -f /tmp/dhcpExpiredLeaseReused ] && [ ! -f /tmp/addressWatchdog.pid ]; then
            logger -p daemon.warning "NETWORK LEASE LOST - DANGEROUS MODE ACTIVATED"
            /Opera_Deploy/networking/addressWatchDog.sh&
        else
            deconfigureInterface
            logger -p daemon.warning "NETWORK DECONFIGURED (L3)"
        fi

        rm -f /tmp/isBound
    ;;

    renew)

        preprocessRoutes

        # Had a renewal we only need to inform framework or reconfigure if
        # settings changed.

        date >> $eventLog
        echo "$interface : RENEW" >> $eventLog
        env | sed -e 's!vendor=!vendor="!' -e '/vendor=/s!$!"!' -e 's!statroute=!statroute="!' -e '/statroute=/s!$!"!' -e 's!statrout2=!statrout2="!' -e '/statrout2=/s!$!"!' -e 's!statrout3=!statrout3="!' -e '/statrout3=/s!$!"!' > $serverParameters
        . $activeParameters

        if [ "$ip" != "$confIp" ] || [ "$subnet" != "$confSubnet" ] || [ "$router" != "$confRouter" ] || [ "$vendor" != "$confVendor" ] || [ "$statroute" != "$confStatroute" ] || [ "$statrout2" != "$confStatrout2" ] || [ "$statrout3" != "$confStatrout3" ]; then
            echo "$interface : RENEW IP/route changed" >> $eventLog

            #delete ip, subnet and router from active parameters
            rm -f tmpfile
            sed -e '/confIp/d' -e '/confSubnet/d' -e '/confRouter/d' -e '/confInterface/d' -e '/confVendor/d' -e '/confStatroute/d' -e '/confStatrout2/d' -e '/confStatrout3/d' $activeParameters > tmpfile
            mv tmpfile $activeParameters
            configureInterface
            configureRouting
            notifyBasicInterfaceUp
            logger -p daemon.notice "NETWORK PARAMETER CHANGED (L3)"
            logger -p daemon.notice < $activeParameters
        fi

        if [ "$sipserver" != "$confSipserver" ]; then
            echo "$interface : RENEW sipserver changed" >> $eventLog
            rm -f tmpfile
            sed -e '/confSipserver/d' $activeParameters  > tmpfile
            mv tmpfile $activeParameters
            notifySipService
            logger -p daemon.notice "NETWORK PARAMETER CHANGED (L3)"
            logger -p daemon.notice < $activeParameters
        fi

        if [ "$ntpsrv" != "$confNtpsrv" ] || [ "$timezone" != "$confTimezone" ]; then
            echo "$interface : RENEW timeserver or timezone changed" >> $eventLog
            rm -f tmpfile
            sed -e '/confNtpsrv/d' -e '/confTimezone/d' $activeParameters  > tmpfile
            mv tmpfile $activeParameters
            notifyTimeService
            logger -p daemon.notice "NETWORK PARAMETER CHANGED (L3)"
            logger -p daemon.notice < $activeParameters
        fi

        if [ "$dns" != "$confDns" ] || [ "$domain" != "$confDomain" ]; then
            echo "$interface : RENEW dns or domain changed" >> $eventLog
            rm -f tmpfile
            sed -e '/confDns/d' -e '/confDomain/d'  $activeParameters  > tmpfile
            mv tmpfile $activeParameters
            configureDns
            logger -p daemon.notice "NETWORK PARAMETER CHANGED (L3)"
            logger -p daemon.notice < $activeParameters
        fi

        cp $serverParameters $backupParameters
    ;;

    bound)
        # Check if lease reuse mode active
        # and shutdown the watchdog

        if [ -f /tmp/addressWatchdog.pid ]; then
            logger -p daemon.warning "NETWORK LEASE OBTAINED - LEASE REUSE MODE DEACTIVATED (L3)"
            # if use expired lease watchdog is running kill it off
            watchdogPid=`cat /tmp/addressWatchdog.pid`
            kill -9 $watchdogPid
            rm -f /tmp/addressWatchdog.pid
        fi

        preprocessRoutes

        # Have been allocated a DHCP lease
        # configure the new lease
        rm -f $activeParameters
        rm -f $serverParameters
        env | sed -e 's!vendor=!vendor="!' -e '/vendor=/s!$!"!' -e 's!statroute=!statroute="!' -e '/statroute=/s!$!"!' -e 's!statrout2=!statrout2="!' -e '/statrout2=/s!$!"!' -e 's!statrout3=!statrout3="!' -e '/statrout3=/s!$!"!' > $serverParameters
        cp $serverParameters $backupParameters
        configureInterface
        configureRouting
        configureDns
        notifyTimeService
        notifySipService

        # File used as flag to select the right path in "deconfig" after "lost lease" sequence:
        #   start udhcp - deconfig - bound - [renew] - leasefail - deconfig
        touch /tmp/isBound

        # Create file to present that a valid lease is available.
        # Will be removed before dhcp release is initiated to to select the right path in "deconfig" for deconfiguring the interface.
        # File is related to OCMS tag dhcp-expired-lease-reused, enum I_EXPIRED_LEASE_REUSED and D_DHCP_EXPIRED_LEASE_REUSED in case of reuse
        touch /tmp/dhcpExpiredLeaseReused

        # Inform TransportMgr that the lease is now again assigned by DHCP
        $testprefix netcmd dhcpConfChange -i $interface -l "false"

        if [ -f $currentVlanPath ]; then
            cp -f $currentVlanPath $savedVlanPath
        else
            rm -f $savedVlanPath
        fi
        if [ -f $currentPortPath ]; then
            cp -f $currentPortPath $savedPortPath
        else
            rm -f $savedPortPath
        fi

        notifyBasicInterfaceUp
        logger -p daemon.notice "NETWORK REINSTATED (L3)"
        logger -p daemon.notice < $activeParameters
    ;;

    leasefail)

        logger -p daemon.warning "DHCP SERVER NOT REACHABLE (L3)"

        # No DHCP server available
        # If we had a lease in the past and we are in lease reuse mode
        # then reuse the old lease data but only if we think
        # the network topology has not changed

        topologyChangeCheck

        if [ "$topologyChanged" -eq "1" ]; then
            logger -p daemon.warning "NO DHCP SERVER - NETWORK TOPOLOGY CHANGE INDICATED (L3)"
        fi

        if [ -f /data/networking/enableDhcpLeaseReuse ] && [ -f /tmp/dhcpExpiredLeaseReused ] && [ -f $backupParameters ] && [ ! -f /tmp/addressWatchdog.pid ] && [ "$topologyChanged" -eq "0" ]; then
            logger -p daemon.warning "NO DHCP SERVER - TRYING TO ACTIVATE LEASE REUSE MODE (L3)"

            rm -f $activeParameters
            rm -f $serverParameters
            cp $backupParameters $serverParameters
            . $backupParameters

            #check the address by arping it
            arping -c 5 -D -q $ip
            if [ "$?" != "0" ]; then
                # Inform TransportMgr that the lease is no longer available because of duplicate ipaddress
                $testprefix netcmd dhcpConfChange -i $interface -l "false"

                rm -f $serverParameters
                logger -p daemon.warning "NO DHCP SERVER - FAILED TO ACTIVATE LEASE REUSE MODE $ip ADDRESS BUSY (L3)"
            else
                configureInterface
                configureRouting
                configureDns
                notifyTimeService
                notifySipService
                notifyBasicInterfaceUp
                logger -p daemon.warning "NO DHCP SERVER - LEASE REUSE MODE ACTIVATED (L3)"

                # Inform TransportMgr that a valid lease is available and a LEASE REUSE MODE is possible after reboot
                # Creates file /tmp/dhcpExpiredLeaseReused during boot and allows to sets the discovered vlan-id
                # to L2 (VVID) / L3 (DISCOVEREDVLAN)

                # Attention: for V2R0 this feature is not released, so set lease reuse to "false"
                # $testprefix netcmd dhcpConfChange -i $interface -l "true"
                $testprefix netcmd dhcpConfChange -i $interface -l "false"

                /Opera_Deploy/networking/addressWatchDog.sh&
            fi
        fi
    ;;

esac

exit 0
