baseInit()
{
if [ "$testing" -eq "1" ]; then

    export PATH="/bin:./"
    testprefix=echo
    installBase=/home/phillipo/tmp

    . $installBase/networking/testParameters.sh

    fwEventPipe="$installBase/networking/${interface}.pipe"
    resolverParameters="$installBase/networking/resolv.conf"

    mkdir -p /tmp/networking

    savedPortPath="$installBase/networking/portDescr"
    currentPortPath="$installBase/portDescr"
    savedVlanPath="$installBase/networking/vlan"
    currentVlanPath="$installBase/vlan"
else

    export PATH="./:/bin:/sbin:/usr/bin:/Opera_Deploy:/data/networking"
    testprefix=
    installBase=/data
    fwEventPipe="/tmp/gpal/ipc/$interface"
    resolverParameters="/etc/resolv.conf"

    savedPortPath="/data/networking/portDescr"
    currentPortPath="/tmp/portDescr"
    savedVlanPath="/data/networking/vlan"
    currentVlanPath="/tmp/vlan"
fi

    serverParameters="$installBase/networking/dhcp.$interface.sh"
    activeParameters="$installBase/networking/active.$interface.sh"
    backupParameters="$installBase/networking/backup.$interface.sh"
    eventLog="$installBase/networking/dhcpEventLog.txt"
    errorLog="$installBase/networking/dhcpErrorLog.txt"
}

topologyChangeCheck()
{
    local savedVlan
    local currentVlan
    local savedPort
    local currentPort

    topologyChanged=0

    # If a transition from a vlan to no vlan or visa-versa that is a topology change

    if [ ! -f $savedVlanPath ] && [ -f $currentVlanPath ]; then
        topologyChanged=1
        return
    fi

    if [ -f $savedVlanPath ] && [ ! -f $currentVlanPath ]; then
        topologyChanged=1
        return
    fi

    # If a transition from having a port descriptor to none or visa-vera that is a topology change

    if [ ! -f $savedPortPath ] && [ -f $currentPortPath ]; then
        topologyChanged=1
        return
    fi

    if [ -f $savedPortPath ] && [ ! -f $currentPortPath ]; then
        topologyChanged=1
        return
    fi

    # We have a VLAN; check if it changed

    if [ -f $savedVlanPath ]; then
        # check for change of VLAN
        savedVlan=`cat $savedVlanPath`
        currentVlan=`cat $currentVlanPath`
        if [ "$savedVlan" != "$currentVlan" ]; then
            topologyChanged=1
            return
        fi
    fi

    # We have a port; check if it changed

    if [ -f $savedPortPath ]; then
        # check for change of VLAN
        savedPort=`cat $savedPortPath`
        currentPort=`cat $currentPortPath`
        if [ "$savedPort" != "$currentPort" ]; then
            topologyChanged=1
            return
        fi
    fi
}

deconfigureInterface()
{
    rm -f $activeParameters
    rm -f $serverParameters
    $testprefix ifconfig $interface 0.0.0.0
    $testprefix netcmd dhcpDeconfig -i $interface
    logger -p daemon.notice "$scriptname: $interface : DOWN"
}

configureInterface()
{
[ -n "$broadcast" ] && BROADCAST="broadcast $broadcast"
[ -n "$subnet" ] && NETMASK="netmask $subnet"
    $testprefix ifconfig $interface $ip $BROADCAST $NETMASK
}

notifyBasicInterfaceUp()
{
    $testprefix netcmd dhcpConfChange -i $interface -b "$ip $subnet $router"
    echo "confInterface=$interface" >> $activeParameters
    echo "confIp=$ip" >> $activeParameters
    echo "confSubnet=$subnet" >> $activeParameters
    echo "confRouter=\"$router\"" >> $activeParameters

    # DON'T MOVE ALWAYS SECOND TO LAST MESSAGE TO NETWORK MANAGER
    # Send vendor options to network manager first
    # this allow us to see if we have been given a discovered VLAN

    if [ -n "$vendor" ] ; then
        $testprefix netcmd dhcpConfChange -i $interface -v "$vendor"
        echo "confVendor=\"$vendor\"" >> $activeParameters
    fi

    # DON'T MOVE - ALWAYS LAST MESSAGE TO NETWORK MANAGER
    # Network manager can then decide to reconfigure the ntwork VLAN
    $testprefix netcmd dhcpBound -i $interface
    logger -p daemon.notice "$scriptname: $interface : UP"
}

preprocessRoutes()
{
    # if classless static route option is present we must ignore router and static route options
    if [ -n "$statrout2" ] ; then
        unset router
        unset statroute
    fi

    # same for microsoft classless static routes
    if [ -n "$statrout3" ] ; then
        unset router
        unset statroute
    fi
}

configureStaticRoutes()
{
    #if odd number of parameters error
    #otherwise try to add destination route entries to table
    if [ `expr $# % 2` -eq  "1" ]; then
        logger -p daemon.err "$scriptname: could not process static route; none added statroute=$*"
        return
    fi

    statRouteWithMask=""

    while [ "$#" -ne "0" ]; do
      mask=`echo $1 | \
        awk -F '.' '($2==0) && ($3==0) && ($4==0) { printf("255.0.0.0") } \
                    ($2!=0) && ($3==0) && ($4==0) { printf("255.255.0.0") } \
                    ($2!=0) && ($3!=0) && ($4==0) { printf("255.255.255.0") }'`
      if [ -z "$mask" ]; then
        route add -host $1 gw $2 dev $interface
        if [ $? = 0 ]; then
          logger -p daemon.notice "$scriptname: route add -host $1 gw $2 dev $interface"
        else
          logger -p daemon.err "$scriptname: \"route add -host $1 gw $2 dev $interface\" failed"
        fi
        # Workaround for buggy NetworkManager::serializerInit
        if [ -z "$statRouteWithMask" ]; then
            statRouteWithMask="$1 $2 255.255.255.255"
        else
            statRouteWithMask="$statRouteWithMask $1 $2 255.255.255.255"
        fi
      else
        route add -net $1 gw $2 netmask $mask dev $interface
        if [ $? = 0 ]; then
          logger -p daemon.notice "$scriptname: route add -net $1 gw $2 netmask $mask dev $interface"
        else
          logger -p daemon.err "$scriptname: \"route add -net $1 gw $2 netmask $mask dev $interface\" failed"
        fi
        # Workaround for buggy NetworkManager::serializerInit
        if [ -z "$statRouteWithMask" ]; then
            statRouteWithMask="$1 $2 $mask"
        else
            statRouteWithMask="$statRouteWithMask $1 $2 $mask"
        fi
      fi
       
      shift 2
    done

    echo "confStatroute=\"$statroute\"" >> $activeParameters

    #now inform the framework of static routes
    #netcmd dhcpConfChange -i $interface -r "$statrouteWithMask"
    netcmd dhcpConfChange -i $interface -r "$statRouteWithMask"
}

configureStaticRoutes2()
{
    #dhcp option 121: static route option
    #if number of parameters are not 3 or 6 error
    #otherwise try to add destination route entries to table
    if [ `expr $# % 3` -ne  "0" ]; then
        logger -p daemon.err "$scriptname: could not process static route; none added statroute-option-121=$*"
        return
    fi

    while [ "$#" -ne "0" ]; do
        route add -net $1 gw $3 netmask $2 dev $interface
        if [ $? = 0 ]; then
          logger -p daemon.notice "$scriptname: route add -net $1 gw $3 netmask $2 dev $interface"
        else
          logger -p daemon.err "$scriptname: \"route add -net $1 gw $3 netmask $2 dev $interface\" failed"
        fi
        shift 3
    done

    echo "confStatrout2=\"$statrout2\"" >> $activeParameters

    #now inform the framework of static routes
    #netcmd dhcpConfChange -i $interface -r "$1 $3 $2"
    netcmd dhcpConfChange -i $interface -r "$1 $3 $2"
}

configureStaticRoutes3()
{
    #dhcp option 249: static route option from MS
    #if number of parameters are not 3 or 6 error
    #otherwise try to add destination route entries to table
    if [ `expr $# % 3` -ne  "0" ]; then
        logger -p daemon.err "$scriptname: could not process static route; none added statroute-option-249=$*"
        return
    fi

    while [ "$#" -ne "0" ]; do
        route add -net $1 gw $3 netmask $2 dev $interface
        if [ $? = 0 ]; then
          logger -p daemon.notice "$scriptname: route add -net $1 gw $3 netmask $2 dev $interface"
        else
          logger -p daemon.err "$scriptname: \"route add -net $1 gw $3 netmask $2 dev $interface\" failed"
        fi
        shift 3
    done

    echo "confStatrout3=\"$statrout3\"" >> $activeParameters

    #now inform the framework of static routes
    #netcmd dhcpConfChange -i $interface -r "$1 $3 $2"
    netcmd dhcpConfChange -i $interface -r "$1 $3 $2"
}

configureRouting()
{
    if [ -n "$router" ] ; then
        logger -p daemon.notice "$scriptname: deleting routes"
        if [ "$testing" -ne "1" ]; then
            while route del default gw 0.0.0.0 dev $interface ; do
                :
            done

            for i in $router ; do
                route add default gw $i dev $interface
            done
        fi
      fi

    if [ -n "$statroute" ]; then
        configureStaticRoutes $statroute
    fi
    if [ -n "$statrout2" ]; then
        configureStaticRoutes2 $statrout2
    fi
    if [ -n "$statrout3" ]; then
        configureStaticRoutes3 $statrout3
    fi
}


configureDns()
{

      if [ -n "$dns" ] ; then
          echo "confDns=\"$dns\"" >> $activeParameters
         $testprefix netcmd dhcpConfChange -i $interface -d "$dns"
      else
         if [ -n "$confDns" ] ; then 
            $testprefix netcmd dhcpConfChange -i $interface -d "$dns"
	 fi
      fi
      if [ -n "$domain" ] ; then
          echo "confDomain=$domain" >> $activeParameters
      	 $testprefix netcmd dhcpConfChange -i $interface -n "$domain"
      else
         if [ -n "$confDomain" ] ; then 
            $testprefix netcmd dhcpConfChange -i $interface -n "$domain"
	 fi
      fi

}

notifyTimeService()
{
    if [ -n "$ntpsrv" ] ; then
        $testprefix netcmd dhcpConfChange -i $interface -t "$ntpsrv"
        logger -p daemon.notice "$scriptname: ntpsrv = \"$ntpsrv\""
        echo "confNtpsrv=\"$ntpsrv\"" >> $activeParameters
    fi
      
    if [ -n "$timezone" ] ; then
        $testprefix netcmd dhcpConfChange -i $interface -o "$timezone"
        logger -p daemon.notice "$scriptname: timeoffset = \"$timezone\""
        echo "confTimezone=$timezone" >> $activeParameters
    fi  

}

notifySipService()
{
      if [ -n "$sipserver" ] ; then
            $testprefix netcmd dhcpConfChange -i $interface -s "$sipserver"
            echo "confSipserver=\"$sipserver\"" >> $activeParameters
      fi
}

isNumeric()
{
    if [ $# -ne 1 ]; then
        return 1
    fi

    case "$1" in
        *[!0-9]*|"" ) return 1;;
        * )  return 0;;
    esac

}


#isNumeric 10
#isNumeric a
#isNumeric 10.54
#isNumeric aaa.333
#isNumeric 333.aa
#isNumeric 23 23
#isNumeric


inRange()
{
    if !(isNumeric "$1"); then
        return 1
    fi
    if [ "$1" -ge "0" ] && [ "$1" -le "255" ]; then
        return 0
    fi

    return 1
}

isIpAddress()
{

    local i

    #echo $1

    if [ $# -ne 1 ]; then
        return 1
    fi

    oldIFS="$IFS"
    IFS=.

    x=0

    for i in $1
    do
        x=`expr $x + 1`
        if !(inRange "$i" );then
            IFS="$oldIFS"
            return 1
        fi
    done

    if [ "$x" -ne "4" ]; then
        IFS="$oldIFS"
        return 1
    fi

    IFS="$oldIFS"
    return 0
}


#isIpAddress 192.168.2.1
#isIpAddress blah
#isIpAddress blash.blah
#isIpAddress
#isIpAddress 192
#isIpAddress -192
#isIpAddress 137.223.237.11
#isIpAddress 255.255.255.255
#isIpAddress 256.256.256.256
