/* sipoption.c
 *
 * Constuct and send SIP DHCP option packets
 *
 * Hsiren Wang May 2004
 *
 */


#include <string.h>
#include "serverpacket.h"
#include "dhcpd.h"
#include "options.h"
#include "common.h"
#include <stdlib.h>
#include <unistd.h>
#include "clientpacket.h"
#include "dhcpc.h"

#ifdef IFX_DHCP_SIP
#include"sipoption.h"

/* handle SIPDHCP event for DHCP client*/
int sip_dhcpc_event_handler(struct sipToDhcpMsg *sip_packet)
{
        uint8_t event;
        struct dhcpMessage *packet;
        uint8_t *option;
        unsigned int i;
        struct sipToDhcpSipServer *sipservers;
        struct sipToDhcpSipServer *siptemp,*temp;
        int len;
     
        event = sip_packet->sipDhcpEvtId;
        packet = sip_packet->dhcpMsg;
        sipservers = NULL;
        siptemp = NULL;
   
        switch(event){
        case SIPDHCP_ENABLE_SIPSERVER_OPTION:
          if(client_config.class != NULL)
          {          
          if(!strncmp(client_config.class+2,SIP_CLIENT_CLASS,sizeof(SIP_CLIENT_CLASS)))
          {
	    DEBUG(LOG_INFO, "sip_dhcpc_event_handler %d %s", SIPDHCP_ENABLE_SIPSERVER_OPTION,SIP_CLIENT_CLASS);
	    add_option_string(packet->options, client_config.class);
          }
	  else if(!strncmp(client_config.class+2,SIP_SERVER_CLASS,sizeof(SIP_SERVER_CLASS)))
          {

          }
          }
          break;
        case SIPDHCP_ADD_SIPSERVER_OPTION_REQUEST_PARAMETER:
          if(client_config.class != NULL)
          {
          if(!strncmp(client_config.class+2,SIP_CLIENT_CLASS,sizeof(SIP_CLIENT_CLASS)))
          {
	    DEBUG(LOG_INFO, "sip_dhcpc_event_handler %d", SIPDHCP_ADD_SIPSERVER_OPTION_REQUEST_PARAMETER);
	    add_simple_option(packet->options, DHCP_SIP,0);
          }
	  else if(!strncmp(client_config.class+2,SIP_SERVER_CLASS,sizeof(SIP_SERVER_CLASS)))
          {

          }
          }
          break;
        case SIPDHCP_NOTIFY_SIPSERVER_LIST:
	  if((option = get_option(packet, DHCP_SIP))!=NULL) {
          if(client_config.class != NULL)
          {
	  if(!strncmp(client_config.class+2,SIP_CLIENT_CLASS,sizeof(SIP_CLIENT_CLASS)))
          {
	    DEBUG(LOG_INFO, "sip_dhcpc_event_handler %d %s", SIPDHCP_NOTIFY_SIPSERVER_LIST,SIP_CLIENT_CLASS);
              len = (option[OPT_LEN]-1)/4;
              if(len > 0)
	      {
                if(option[OPT_LEN+1] == OPTION_IPV4_ADDR)
		{
                  for(i=0;i<len;i++)
		  {
                    if(!sipservers)
		    {
                      sipservers = (struct sipToDhcpSipServer *)xmalloc(sizeof(struct sipToDhcpSipServer));  
                      siptemp = sipservers;
                      siptemp->next = NULL;
                    }
                    else
		    {
                      siptemp->next = (struct sipToDhcpSipServer *)xmalloc(sizeof(struct sipToDhcpSipServer));                      
                      siptemp = siptemp->next;
                      siptemp->next = NULL;
                    }
                    siptemp->addr = *((uint32_t*)&option[OPT_LEN+2+i*4]);
                    siptemp->name = NULL;
                  }
	          DEBUG(LOG_INFO, "sip_dhcpc_event_handler %d %s,sipserver ipv4 list:",
                        SIPDHCP_NOTIFY_SIPSERVER_LIST,SIP_CLIENT_CLASS,option);
                  /* TODO integrate with sip client using sipservers */
	          for(siptemp=sipservers;siptemp!=NULL;siptemp=temp)
		  {
                    temp = siptemp->next;
                    free(siptemp);
		  }
                }
                else if(option[OPT_LEN+1] == OPTION_DOMAIN_NAME)
		{
                } 
              }
	    }  
	    else if(!strncmp(client_config.class+2,SIP_SERVER_CLASS,sizeof(SIP_SERVER_CLASS)))
            {
	      DEBUG(LOG_INFO, "sip_dhcpc_event_handler %d %s", SIPDHCP_NOTIFY_SIPSERVER_LIST,SIP_SERVER_CLASS);
            }
          }
          }
          break;
        case SIPDHCP_DISABLE_SIPSERVER_OPTION:
          break;
        default:
          break;
        }
	return 0;
}


/* handle SIPDHCP event for DHCP server*/
int sip_dhcpd_event_handler(struct sipToDhcpMsg *sip_packet,
                              struct dhcpMessage *oldpacket,struct dhcpOfferedAddr *lease)
{
        uint8_t event;
        struct dhcpMessage *packet;
        uint8_t *vendor;
        unsigned int i;
        struct sipToDhcpSipServer *sipservers;
        struct sipToDhcpSipServer *siptemp,*temp;
        int len;
        uint8_t *addr;

        event = sip_packet->sipDhcpEvtId;
        packet = sip_packet->dhcpMsg;
        sipservers = NULL;
        siptemp = NULL;
   
        switch(event){
        case SIPDHCP_REGISTER_SIPSERVER_CLASS:
	  if ((vendor = get_option(oldpacket, DHCP_VENDOR))!=NULL) {
	    if (!strncmp(vendor, SIP_SERVER_CLASS, sizeof(SIP_SERVER_CLASS))) {
	      DEBUG(LOG_INFO, "sip_dhcpd_event_handler %d %s", SIPDHCP_REGISTER_SIPSERVER_CLASS,SIP_SERVER_CLASS);
              if(lease)
                lease->vclass = CLASS_SIPSERVER;  	      
            }
	    else if (!strncmp(vendor, SIP_CLIENT_CLASS, sizeof(SIP_CLIENT_CLASS))) {
	      DEBUG(LOG_INFO, "sip_dhcpd_event_handler %d %s", SIPDHCP_REGISTER_SIPSERVER_CLASS,SIP_CLIENT_CLASS);
            }
	  }
          break;
        case SIPDHCP_ADD_SIPSERVER_OPTION:
	  if ((vendor = get_option(oldpacket, DHCP_VENDOR))!=NULL) {
	    if (!strncmp(vendor, SIP_SERVER_CLASS, sizeof(SIP_SERVER_CLASS))) {
	      DEBUG(LOG_INFO, "sip_dhcpd_event_handler %d %s", SIPDHCP_ADD_SIPSERVER_OPTION,SIP_SERVER_CLASS);
            }
	    else if (!strncmp(vendor, SIP_CLIENT_CLASS, sizeof(SIP_CLIENT_CLASS))) {
	      DEBUG(LOG_INFO, "sip_dhcpd_event_handler %d %s", SIPDHCP_ADD_SIPSERVER_OPTION,SIP_CLIENT_CLASS);
	      len = 0;
              for (i = 0; i < server_config.max_leases; i++) {
		if (leases[i].yiaddr != 0) {
                  if(leases[i].vclass == CLASS_SIPSERVER){
                    if(!sipservers)
		    {
                      sipservers = (struct sipToDhcpSipServer *)xmalloc(sizeof(struct sipToDhcpSipServer));  
                      siptemp = sipservers;
                      siptemp->next = NULL;
                    }
                    else
		    {
                      siptemp->next = (struct sipToDhcpSipServer *)xmalloc(sizeof(struct sipToDhcpSipServer));                      
                      siptemp = siptemp->next;
                      siptemp->next = NULL;
                    }
                    siptemp->addr = leases[i].yiaddr;
                    siptemp->name = NULL;
                    len++;
                  }
                }
              }
              if(len > 0)
	      {
                siptemp = sipservers;
                addr = (uint8_t *)xmalloc(len*4+1+2);
                addr[OPT_CODE] = DHCP_SIP;
                addr[OPT_LEN] = len*4+1;
                addr[OPT_LEN+1] = OPTION_IPV4_ADDR;
                for(i=0;i<len;i++)
	        {
                  strncpy(addr+3+i*4,(uint8_t *)&siptemp->addr,4);
                  siptemp = siptemp->next;
	        }
                add_option_string(packet->options,addr);
	        DEBUG(LOG_INFO, "sip_dhcpd_event_handler %d %s,sipserver ipv4 list:",
                        SIPDHCP_ADD_SIPSERVER_OPTION,SIP_CLIENT_CLASS,addr);
	        for(siptemp=sipservers;siptemp!=NULL;siptemp=temp)
		  {
                    temp = siptemp->next;
                    free(siptemp);
		  }
              }
	    }  
          }
          break;
        case SIPDHCP_DEREGISTER_SIPSERVER_CLASS:
          break;
        default:
          break;
        }
	return 0;
}



#endif

